using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Interop02
{
   /// <summary>
   ///   Subclassed version of Control that does everything necessary to become an Active-X control
   /// </summary>
   [ClassInterface(ClassInterfaceType.AutoDual)]
   public class ActiveXControl: System.Windows.Forms.UserControl
   {
      [ComRegisterFunction]
      static void ComRegister(Type t)
      {
         string keyName = @"CLSID\" + t.GUID.ToString("B");
         using (RegistryKey key = Registry.ClassesRoot.OpenSubKey(keyName, true))
         {
            key.CreateSubKey("Control").Close();
            using (RegistryKey subkey = key.CreateSubKey("MiscStatus"))
            {
               subkey.SetValue("", "131457");
            }
            using (RegistryKey subkey = key.CreateSubKey("TypeLib"))
            {
               Guid libid = Marshal.GetTypeLibGuidForAssembly(t.Assembly);
               subkey.SetValue("", libid.ToString("B"));
            }
            using (RegistryKey subkey = key.CreateSubKey("Version"))
            {
               Version ver = t.Assembly.GetName().Version;
               string version = string.Format("{0}.{1}", ver.Major, ver.Minor);
               if (version == "0.0")
                  version = "1.0";
               subkey.SetValue("", version);
            }
         }
      }

      [ComUnregisterFunction]
      static void ComUnregister(Type t)
      {
         Registry.ClassesRoot.DeleteSubKeyTree(@"CLSID\" + t.GUID.ToString("B"));
      }
   }
}
